package AndroidTraj.apk;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.ArrayList;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.method.ScrollingMovementMethod;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

public class AndroidTraj extends Activity {

	private boolean IN_PROGRESS = false;

	SharedPreferences preferences;
	private TextView tvTerm;
	private Button btnConnect;
	private Button btnClose;
	private Button btnOGPS;

	private int coSize = 0;
	ArrayList<Double> Coordinates = new ArrayList<Double>();
	Double[] trajectory = null;

	private LocationManager locationManager = null;
	private LocationListener locationListener = null;
	private File outFile = null;
	private File inFile = null;
	private Socket connection = null;
	BufferedWriter writer = null;
	BufferedReader reader = null;

	private String serverAddress = new String();
	private String epsilon = new String();
	private String custFilePath = new String();
	private String outpFileloc = new String();

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.main);

		initControls(this);

		// ---use the LocationManager class to obtain GPS locations---
		locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
		locationListener = new MyLocationListener();

		tvTerm.setText("\nAplication started...\n");
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.menu, menu);
		return true;
	}

	// This method is called once the menu is selected
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		// We have only one menu option
		case R.id.appPref:
			// Launch Preference activity
			Intent i = new Intent(AndroidTraj.this, Preferences.class);
			startActivity(i);
			// A toast is a view containing a quick little message for the user.
			Toast.makeText(AndroidTraj.this,
					"Here are the application's preferences.",
					Toast.LENGTH_LONG).show();
			break;

		}
		return true;
	}

	private void initControls(AndroidTraj a) {

		tvTerm = (TextView) findViewById(R.id.tvTerm);
		tvTerm.setMovementMethod(new ScrollingMovementMethod());
		btnConnect = (Button) findViewById(R.id.serverConnection);
		btnClose = (Button) findViewById(R.id.appClose);
		btnOGPS = (Button) findViewById(R.id.btnOGPS);
		preferences = PreferenceManager.getDefaultSharedPreferences(a);

		btnConnect.setOnClickListener(new Button.OnClickListener() {
			public void onClick(View v) {

				serverAddress = preferences.getString("serverAdd", "n/a");
				if (serverAddress.equals("n/a") || serverAddress.equals("")) {
					tvTerm
							.append("\nProvide server's ip from menu -> preferences.");
					return;
				}

				epsilon = preferences.getString("epsilon", "n/a");
				if (epsilon.equals("n/a") || epsilon.equals("")) {
					tvTerm
							.append("\nEpsilon not provided. Using 2 as default.");
					epsilon = "2";
				}

				if (IN_PROGRESS == false) {
					IN_PROGRESS = true;

					if (btnOGPS.getText().equals("Stop GPS")) {
						try {
							if (writer != null)
								writer.close();
						} catch (Exception e) {
							tvTerm.setText("writer Exception:" + e);
						}						
						locationManager.removeUpdates(locationListener);
						btnOGPS.setText("Start GPS");
					}

					serverConnect();
					IN_PROGRESS = false;
				}
			}
		});

		btnClose.setOnClickListener(new Button.OnClickListener() {
			public void onClick(View v) {
				appClose();
			}
		});

		btnOGPS.setOnClickListener(new Button.OnClickListener() {
			public void onClick(View v) {

				outpFileloc = preferences.getString("outpFile", "n/a");
				
				if (outpFileloc.equals("n/a") || outpFileloc.equals("")) {
					tvTerm.append("Provide path/filename for output file.\n(menu preferences)\n\n");
					return;
				}
				
				
				if (btnOGPS.getText().equals("Start GPS")) {

					// initialize the coordinates file
					try {
						outFile = new File(outpFileloc);
						writer = new BufferedWriter(new FileWriter(outFile));
					} catch (Exception e) {
						tvTerm.setText("Error when opening output file:\n" + e.getMessage());
					}
					
					try {
						locationManager.requestLocationUpdates(
								LocationManager.GPS_PROVIDER, 0, 0,
								locationListener);
					} catch (Exception e) {
						tvTerm.setText("Unexpected Error on GPS service:\n"
								+ e.getMessage());
					}
					btnOGPS.setText("Stop GPS");
					tvTerm
							.setText("\nGPS tracking started...\nCollecting Position Information...");
				} else {
					
					try {
						if (writer != null)
							writer.close();
					} catch (Exception e) {
						tvTerm.setText("writer Exception:" + e);
					}
					
					locationManager.removeUpdates(locationListener);
					btnOGPS.setText("Start GPS");
					tvTerm
							.setText("\nGPS tracking stoped...\nClick Connect to server to upload data.");

				}
			}
		});
	}

	private void appClose() {
		finish();
		System.exit(0);
	}

	private void serverConnect() {
		
		String line = new String();
		String host = new String();
		String buf = new String();
		String ClientTRAJ = new String();
		String[] traj = null;
		InputStream inStream = null;
		DataInputStream inDataStream = null;
		OutputStream outStream = null;
		DataOutputStream outDataStream = null;
		int i = 0;

		// open the coordinates file
		try {
			custFilePath="";
			custFilePath = preferences.getString("custFile", "n/a");
			if (custFilePath.equals("n/a") || custFilePath.equals("")) {
				tvTerm.setText("\nUsing default trajectory file (GPS data).\n");
				inFile = new File(outpFileloc);
			} else {
				tvTerm.setText("\nUsing trajectory file: " + custFilePath + ".\n");
				inFile = new File(custFilePath);
			}
			
			reader = new BufferedReader(new FileReader(inFile));
			
		} catch (Exception e) {
			tvTerm.setText("Error when opening file:\n" + e.getMessage());
		}

		ClientTRAJ = "";
		
		// get the Latitude from file and store it to an array
		try {
			while ((line = reader.readLine()) != null) {
				Coordinates.add(Double.parseDouble(line));
				ClientTRAJ += Coordinates.get(i++) + " ";
			}
		} catch (Exception e) {
			tvTerm.append("Exception while reading from file:\n"
					+ e.getMessage());
		}

		try {
			// CONNECT TO SERVER
			host = serverAddress;
			connection = new Socket(host, 8080);
			tvTerm.append("Socket created.\nConnecting to server on " + host
					+ "...\n");

			// INITIALIZE THE IN AND OUT STREAMS
			inStream = connection.getInputStream();
			inDataStream = new DataInputStream(inStream);
			outStream = connection.getOutputStream();
			outDataStream = new DataOutputStream(outStream);

			// GET THE +OK READY FROM SERVER
			buf = "";
			buf = inDataStream.readUTF();
			tvTerm.append("Server Response: " + buf + "\n");

			// SEND A +OK REPLY TO SERVER
			outDataStream.writeUTF("+OK READY");

			// WAIT FOR THE SERVER TO SEND ITS QUERY
			buf = "";
			buf = inDataStream.readUTF();
			tvTerm.append("Received server's Query (TRAJ).\n");
			traj = buf.split(",");
			trajectory = new Double[traj.length];

			for (int j = 0; j < traj.length; j++)
				trajectory[j] = Double.parseDouble(traj[j]);

			// CALCULATE THE UB
			tvTerm.append("Calculating UB...\n");

			// SEND THE UB TO SERVER
			outDataStream.writeUTF("+OK UB "
					+ UBLCSS.upperBound(trajectory, Coordinates, Integer.parseInt(epsilon)));

			// WAIT FOR A SERVER ANS
			tvTerm.append("Waiting for server reply...\n");
			buf = "";
			buf = inDataStream.readUTF();
			tvTerm.append("Server Response: " + buf + "\n");

			// IF +OK RETR TRAJ SEND THE Coordinates[] ARRAY
			if (buf.equals("+OK SEND TRAJ")) {
				tvTerm.append("Sending Trajectory to server...\n");
				outDataStream.writeUTF("+OK TRAJ " + ClientTRAJ);
			}
			// ELSE CLOSE THE PROGRAM
			// ending and closing the application
			connection.close();
			tvTerm.append("Done.\nConnection closed.\n");

		} catch (Exception e) {
			tvTerm.setText("Error:\n" + e.getMessage());
			IN_PROGRESS = false;
		} // end try
	}

	private class MyLocationListener implements LocationListener {

		public void onLocationChanged(Location loc) {

			double latidute;

			if (loc != null) {
				try {
					latidute = loc.getLatitude();
					writer.append(latidute + "\n");
					coSize++;
					Toast.makeText(getBaseContext(),
							"Latitude (X): " + latidute + "\n",
							Toast.LENGTH_LONG).show();
				} catch (Exception e) {
					tvTerm.setText("Exception in onLocationChanged(): "
							+ e.getMessage());
					setContentView(tvTerm);
				}
			}
		}

		public void onProviderDisabled(String provider) {}

		public void onProviderEnabled(String provider) {}

		public void onStatusChanged(String provider, int status, Bundle extras) {}
	}
}